@@ -1,27 +1,37 @@
-VERSION = '0.0.9'
+VERSION = '0.0.12'
 
+import ast
 import collections
 from os import path
 from box import ConfigBox
 from collections import OrderedDict
+from contextlib import contextmanager
 
 class FormatNotSupported(Exception):
 	pass
 
+class UnknownCastType(TypeError):
+	pass
+
 class Loader(object):
 
-	def __init__(self, cfile, with_profile, case_sensitive):
-		self.with_profile   = with_profile
-		self.case_sensitive = case_sensitive
-		self.config         = self.prepare(self.load(cfile))
+	@staticmethod
+	def typeCast(val):
+		try:
+			t, v = val.split(':', 1)
+		except (AttributeError, ValueError):
+			# AttributeError: 1.split, None.split
+			# ValueError: t, v = 'a'.split(':', 1)
+			return val
+		if t == 'py':
+			return ast.literal_eval(v)
+		if t == 'csv':
+			return str(v).split(',')
+		raise UnknownCastType(t + ', currently supported: py and csv.')
 
-	def prepare(self, config):
-		if self.case_sensitive:
-			return config
-		elif self.with_profile:
-			return {key.upper(): {k.upper():v for k, v in val.items()} for key, val in config.items()}
-		else:
-			return {key.upper(): val for key, val in config.items()}
+	def __init__(self, cfile, with_profile):
+		self.with_profile = with_profile
+		self.config       = self.load(cfile)
 	
 	def load(self, cfile):
 		pass
@@ -44,14 +54,15 @@ class IniLoader(Loader):
 		config.read(cfile)
 
 		ret = {sec: dict(config.items(sec)) for sec in config.sections()}
-		defaults = config.defaults()
+		defaults = config.defaults() # section DEFAULT
 		defaults.update(ret.get('default', {}))
 		ret['default'] = defaults
 		
 		if not self.with_profile:
-			return defaults
+			# only default session is loaded
+			return {key: Loader.typeCast(val) for key, val in defaults.items()}
 
-		return ret
+		return {key: {k: Loader.typeCast(v) for k, v in val.items()} for key, val in ret.items()}
 
 class EnvLoader(Loader):
 
@@ -70,7 +81,7 @@ class EnvLoader(Loader):
 		config = DotEnv(cfile).dict()
 		
 		if not self.with_profile:
-			return config
+			return {key: Loader.typeCast(val) for key, val in config.items()}
 			
 		ret = {}
 		for key, val in config.items():
@@ -78,12 +89,11 @@ class EnvLoader(Loader):
 				continue
 			profile, realkey = key.split('_', 1)
 			if profile not in ret:
-				ret[profile] = {realkey: val}
+				ret[profile] = {realkey: Loader.typeCast(val)}
 			else:
-				ret[profile][realkey] = val
+				ret[profile][realkey] = Loader.typeCast(val)
 		return ret
 
-
 class OsEnvLoader(Loader):
 
 	def load(self, cfile):
@@ -98,16 +108,16 @@ class OsEnvLoader(Loader):
 			key = key[len(prefix):]
 
 			if not self.with_profile:
-				config[key] = val
+				config[key] = Loader.typeCast(val)
 				continue
 
 			if '_' not in key:
 				continue
 			profile, realkey = key.split('_', 1)
 			if profile not in config:
-				config[profile] = {realkey: val}
+				config[profile] = {realkey: Loader.typeCast(val)}
 			else:
-				config[profile][realkey] = val
+				config[profile][realkey] = Loader.typeCast(val)
 		return config
 
 class YamlLoader(Loader):
@@ -183,10 +193,10 @@ class Config(ConfigBox):
 	
 	def __init__(self, *args, **kwargs):
 		self.__dict__['_protected'] = dict(
-			with_profile   = kwargs.pop('with_profile', True),
-			case_sensitive = kwargs.pop('case_sensitive', False),
-			profile        = 'default',
-			cached         = OrderedDict()
+			with_profile = kwargs.pop('with_profile', True),
+			profile      = 'default',
+			prevprofile  = None,
+			cached       = OrderedDict()
 		)
 		super(Config, self).__init__(*args, **kwargs)
 
@@ -197,7 +207,6 @@ class Config(ConfigBox):
 	def _load(self, *names):
 		cached         = self._protected['cached']
 		with_profile   = self._protected['with_profile']
-		case_sensitive = self._protected['case_sensitive']
 		profile        = self._protected['profile']
 		for name in names:
 			ext = 'dict' if isinstance(name, dict) else name.rpartition('.')[2]
@@ -205,66 +214,63 @@ class Config(ConfigBox):
 				raise FormatNotSupported(ext)
 			if ext == 'dict':
 				if repr(name) not in cached:
-					cached[repr(name)] = DictLoader(name, with_profile, case_sensitive).config
+					cached[repr(name)] = DictLoader(name, with_profile).config
 				name = repr(name)
 			else:
 				# maybe hash the name?
 				if name not in cached:
-					cached[name] = Loaders[ext](name, with_profile, case_sensitive).config
+					cached[name] = Loaders[ext](name, with_profile).config
 				else:
 					# change the position of the configuration
 					cached[name] = cached.pop(name)
 
 			if with_profile:
-				self.update(cached[name].get(profile if case_sensitive else profile.upper(), {}))
+				self.update(cached[name].get(profile, {}))
 			else:
 				self.update(cached[name])
 
-	def copy(self, profile = 'default'):
-		ret = self.__class__(with_profile = self._protected['with_profile'], case_sensitive = self._protected['case_sensitive'], **self)
-		ret.__dict__['_protected']['profile'] = profile
-		ret.__dict__['_protected']['cached']  = self._protected['cached']
+	def copy(self, profile = None):
+		ret = self.__class__(with_profile = self._protected['with_profile'], **self)
+		ret._protected['profile'] = profile or self._profile
+		ret._protected['cached']  = self._protected['cached']
 		return ret
 
 	def clear(self):
 		super(Config, self).clear()
 		self._protected['cached'] = OrderedDict()
 
-	def __getattr__(self, item):
-		if self._protected['case_sensitive']:
-			return super(Config, self).__getattr__(item)
-		return super(Config, self).__getattr__(item.upper())
+	@property
+	def _protected(self):
+		return self.__dict__['_protected']
 
-	def __getitem__(self, item, _ignore_default = False):
-		if self._protected['case_sensitive']:
-			return super(Config, self).__getitem__(item, _ignore_default)
-		return super(Config, self).__getitem__(item.upper(), _ignore_default)
-	
-	def _use(self, profile = 'default', raise_exc = False):
+	@property
+	def _profile(self):
+		return self._protected['profile']
+
+	@_profile.setter
+	def _profile(self, profile):
+		self._protected['profile'] = profile
+
+	def _revert(self):
+		if not self._protected['prevprofile']:
+			return
+		self._use(self._protected['prevprofile'])
+
+
+	def _use(self, profile = 'default', raise_exc = False, copy = False):
 		if not self._protected['with_profile']:
 			raise ValueError('Unable to switch profile, this configuration is set without profile.')
 
-		if not self._protected['case_sensitive']:
-			profile = profile.upper()
+		if profile == self._profile:
+			return self.copy(profile) if copy else None
 
-		if profile == self._protected['profile']:
-			return self.copy(profile)
+		self._protected['prevprofile'] = self._profile
+		if self._profile != 'default':
+			super(Config, self).clear()
 
-		if self._protected['case_sensitive']:
-			if self._protected['profile'] != 'default':
-				super(Config, self).clear()
-		else:
-			if self._protected['profile'].upper() != 'DEFAULT':
-				super(Config, self).clear()
-
-		if self._protected['case_sensitive']:
-			if profile != 'default':
-				# load default first
-				self._use()
-		else:
-			if profile != 'DEFAULT':
-				# load default first
-				self._use()
+		if profile != 'default':
+			# load default first
+			self._use()
 		
 		hasprofile = False
 		for conf in self._protected['cached'].values():
@@ -276,9 +282,14 @@ class Config(ConfigBox):
 		if raise_exc and not hasprofile:
 			raise NoSuchProfile('Config has no such profile: %s' % profile)
 
-		self._protected['profile'] = profile
+		self._profile = profile
 
-		return self.copy(profile)
+		return self.copy(profile) if copy else None
+
+	@contextmanager
+	def _with(self, profile = 'default', raise_exc = False):
+		yield self._use(profile, raise_exc, copy = True)
+		self._revert()
 
 config = Config()
 
